module Test where

open import Algebra using (Op₂; Magma; Ring)
open import Data.Product     using (_×_; ∃) 
open import Data.Sum         using (_⊎_)
open import Function         using (_∘_) 
open import Level            using (suc; _⊔_)
open import Relation.Binary  using (Rel; Setoid)
open import Relation.Nullary using (¬_)
open import Relation.Unary   using (Pred)


module OfMagma {α α=} (M : Magma α α=)
  where
  open Magma M using (Carrier; _≈_; _∙_; setoid)

  LeftQuotient : Carrier → Carrier → Set (α ⊔ α=)
  LeftQuotient a b =  ∃ (\q → a ≈ (q ∙ b))

  _∣_ _∤_ :  Rel Carrier (α ⊔ α=)
  x ∣ y =  LeftQuotient y x

  _∤_ x =  ¬_ ∘ _∣_ x


module OfRing {α α=} (R : Ring α α=)
  where
  open Ring R using (Carrier; _≈_; 1#; _*_; *-magma)
  open OfMagma *-magma using (_∣_; _∤_)

  IsPrime :  Pred Carrier _  
  IsPrime p =  p ∤ 1#  ×  (∀ {x y} → (p ≈ x * y) → x ∣ 1# ⊎ y ∣ 1#)

  Prime∣split :  Set _ 
  Prime∣split =  {p a b : Carrier} → IsPrime p → p ∣ (a * b) → p ∣ a ⊎ p ∣ b


module _ {α α=} (R : Ring α α=)
  where            
  open Ring R          using (Carrier; _*_; *-magma) 
  open OfMagma *-magma using (_∣_)
  open OfRing R        using (IsPrime; Prime∣split)

  test : (p x y : Carrier) → IsPrime p → p ∣ (x * y) → (p ∣ x) ⊎ (p ∣ y)
  test p x y p-prime p∣xy =  Prime∣split p-prime p∣xy
